@@ -12,16 +12,22 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import glob
 import json
+import os
 import sys
 
 from cliff.command import Command
+from cliff.hooks import CommandHook
 
 from kayobe import ansible
 from kayobe import kolla_ansible
 from kayobe import utils
 from kayobe import vault
 
+# This is set to an arbitrary large number to simplify the sorting logic
+DEFAULT_SEQUENCE_NUMBER = sys.maxsize
+
 
 def _build_playbook_list(*playbooks):
     """Return a list of names of playbook files given their basenames."""
@@ -144,6 +150,73 @@ class KollaAnsibleMixin(object):
         return kolla_ansible.run_seed(*args, **kwargs)
 
 
+def _split_hook_sequence_number(hook):
+    parts = hook.split("-", 1)
+    if len(parts) < 2:
+        return (DEFAULT_SEQUENCE_NUMBER, hook)
+    try:
+        return (int(parts[0]), parts[1])
+    except ValueError:
+        return (DEFAULT_SEQUENCE_NUMBER, hook)
+
+
+class HookDispatcher(CommandHook):
+    """Runs custom playbooks before and after a command"""
+
+# Order of calls: get_epilog, get_parser, before, after
+
+    def __init__(self, *args, **kwargs):
+        self.command = kwargs["command"]
+        self.logger = self.command.app.LOG
+        cmd = self.command.cmd_name
+        # Replace white space with dashes for consistency with ansible
+        # playbooks. Example cmd: kayobe control host bootstrap
+        self.name = "-".join(cmd.split())
+
+    def get_epilog(self):
+        pass
+
+    def get_parser(self, prog_name):
+        pass
+
+    def _find_hooks(self, config_path, target):
+        name = self.name
+        path = os.path.join(config_path, "hooks", name, "%s.d" % target)
+        self.logger.debug("Discovering hooks in: %s" % path)
+        if not os.path.exists:
+            return []
+        hooks = glob.glob(os.path.join(path, "*.yml"))
+        self.logger.debug("Discovered the following hooks: %s" % hooks)
+        return hooks
+
+    def hooks(self, config_path, target):
+        hooks = self._find_hooks(config_path, target)
+        # Hooks can be prefixed with a sequence number to adjust running order,
+        # e.g 10-my-custom-playbook.yml. Sort by sequence number.
+        hooks = sorted(hooks, key=_split_hook_sequence_number)
+        # Resolve symlinks so that we can reference roles.
+        hooks = [os.path.realpath(hook) for hook in hooks]
+        return hooks
+
+    def run_hooks(self, parsed_args, target):
+        config_path = parsed_args.config_path
+        hooks = self.hooks(config_path, target)
+        if hooks:
+            self.logger.debug("Running hooks: %s" % hooks)
+            self.command.run_kayobe_playbooks(parsed_args, hooks)
+
+    def before(self, parsed_args):
+        self.run_hooks(parsed_args, "pre")
+        return parsed_args
+
+    def after(self, parsed_args, return_code):
+        if return_code == 0:
+            self.run_hooks(parsed_args, "post")
+        else:
+            self.logger.debug("Not running hooks due to non-zero return code")
+        return return_code
+
+
 class ControlHostBootstrap(KayobeAnsibleMixin, KollaAnsibleMixin, VaultMixin,
                            Command):
     """Bootstrap the Kayobe control environment.
@@ -256,7 +329,8 @@ class PlaybookRun(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, parsed_args.playbook)
 
 
-class KollaAnsibleRun(KollaAnsibleMixin, VaultMixin, Command):
+class KollaAnsibleRun(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
+                      Command):
     """Run a Kolla Ansible command.
 
     Allows a single kolla-ansible command to be run. For advanced users only.
@@ -339,8 +413,9 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Configure user accounts, group associations, and authorised SSH keys.
     * Configure the host's network interfaces.
     * Set sysctl parameters.
-    * Configure NTP.
+    * Configure timezone.
     * Optionally, configure software RAID arrays.
+    * Optionally, configure encryption.
     * Configure LVM volumes.
     * Configure the host as a libvirt hypervisor.
     """
@@ -373,12 +448,12 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
 
         playbooks = _build_playbook_list(
             "ssh-known-host", "kayobe-ansible-user",
-            "yum", "dnf", "pip", "kayobe-target-venv")
+            "dnf", "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "dev-tools", "network", "sysctl", "ntp",
-            "mdadm", "lvm", "seed-hypervisor-libvirt-host")
+            "users", "dev-tools", "network", "sysctl", "timezone",
+            "mdadm", "luks", "lvm", "seed-hypervisor-libvirt-host")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
 
@@ -495,8 +570,9 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Set sysctl parameters.
     * Configure IP routing and source NAT.
     * Disable bootstrap interface configuration.
-    * Configure NTP.
+    * Configure timezone.
     * Optionally, configure software RAID arrays.
+    * Optionally, configure encryption.
     * Configure LVM volumes.
     * Optionally, create a virtualenv for kolla-ansible.
     * Configure a user account for kolla-ansible.
@@ -516,23 +592,6 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Configuring seed host OS")
 
-        # Query some kayobe ansible variables.
-        # Explicitly request the dump-config tag to ensure this play runs even
-        # if the user specified tags.
-        hostvars = self.run_kayobe_config_dump(parsed_args, hosts="seed",
-                                               tags="dump-config")
-        if not hostvars:
-            self.app.LOG.error("No hosts in the seed group")
-            sys.exit(1)
-        hostvars = list(hostvars.values())[0]
-        ansible_user = hostvars.get("kayobe_ansible_user")
-        if not ansible_user:
-            self.app.LOG.error("Could not determine kayobe_ansible_user "
-                               "variable for seed host")
-            sys.exit(1)
-        python_interpreter = hostvars.get("ansible_python_interpreter")
-        kolla_target_venv = hostvars.get("kolla_ansible_target_venv")
-
         # Allocate IP addresses.
         playbooks = _build_playbook_list("ip-allocation")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
@@ -540,44 +599,25 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Run kayobe playbooks.
         playbooks = _build_playbook_list(
             "ssh-known-host", "kayobe-ansible-user",
-            "yum", "dnf", "pip", "kayobe-target-venv")
+            "dnf", "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "ip-routing", "snat", "disable-glean", "ntp", "mdadm",
-            "lvm", "docker-devicemapper")
+            "sysctl", "ip-routing", "snat", "disable-glean", "timezone",
+            "mdadm", "luks", "lvm", "docker-devicemapper",
+            "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
 
         self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
         # Run kolla-ansible bootstrap-servers.
-        # This command should be run as the kayobe ansible user because at this
-        # point the kolla user may not exist.
-        extra_vars = {"ansible_user": ansible_user}
-        if python_interpreter:
-            # Use the kayobe virtualenv, as this is the executing user.
-            extra_vars["ansible_python_interpreter"] = python_interpreter
-        elif kolla_target_venv:
-            # Override the kolla-ansible virtualenv, use the system python
-            # instead.
-            extra_vars["ansible_python_interpreter"] = "/usr/bin/python"
-        if kolla_target_venv:
-            # Specify a virtualenv in which to install python packages.
-            extra_vars["virtualenv"] = kolla_target_venv
-        self.run_kolla_ansible_seed(parsed_args, "bootstrap-servers",
-                                    extra_vars=extra_vars)
-
-        # Re-run the Pip role after we've bootstrapped the Kolla user
-        extra_vars = {}
-        kolla_ansible_user = hostvars.get("kolla_ansible_user")
-        extra_vars["pip_applicable_users"] = [kolla_ansible_user]
+        self.run_kolla_ansible_seed(parsed_args, "bootstrap-servers")
 
         # Run final kayobe playbooks.
         playbooks = _build_playbook_list(
-            "pip", "kolla-target-venv", "kolla-host", "docker")
-        self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars, limit="seed")
+            "kolla-host", "docker")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
 
         # Optionally, deploy a Docker Registry.
         playbooks = _build_playbook_list("docker-registry")
@@ -649,12 +689,12 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
                         Command):
     """Deploy the seed services.
 
+    * Deploys user defined containers
     * Configures kolla-ansible.
     * Configures the bifrost service.
     * Deploys the bifrost container using kolla-ansible.
     * Builds disk images for the overcloud hosts using Diskimage Builder (DIB).
     * Performs a workaround in the overcloud host image to fix resolv.conf.
-    * Performs a workaround in the overcloud host image to update cloud-init
     * Configures ironic inspector introspection rules in the bifrost inspector
       service.
     * When enabled, configures a Bare Metal Provisioning (BMP) environment for
@@ -663,13 +703,15 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Deploying seed services")
+        playbooks = _build_playbook_list(
+            "seed-deploy-containers")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
         self.generate_kolla_ansible_config(parsed_args, service_config=False,
                                            bifrost_config=True)
 
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
-            "overcloud-host-image-workaround-cloud-init",
             "seed-introspection-rules",
             "dell-switch-bmp")
         self.run_kayobe_playbooks(parsed_args, playbooks)
@@ -679,13 +721,13 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
                          Command):
     """Upgrade the seed services.
 
+    * Deploys user defined containers
     * Configures kolla-ansible.
     * Configures the bifrost service.
     * Prepares the bifrost service for an upgrade.
     * Deploys the bifrost container using kolla-ansible.
     * Builds disk images for the overcloud hosts using Diskimage Builder (DIB).
     * Performs a workaround in the overcloud host image to fix resolv.conf.
-    * Performs a workaround in the overcloud host image to update cloud-init
     * Configures ironic inspector introspection rules in the bifrost inspector
       service.
     * When enabled, configures a Bare Metal Provisioning (BMP) environment for
@@ -694,6 +736,9 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Upgrading seed services")
+        playbooks = _build_playbook_list(
+            "seed-deploy-containers")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
         self.generate_kolla_ansible_config(parsed_args, service_config=False,
                                            bifrost_config=True)
 
@@ -703,7 +748,6 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kolla_ansible_seed(parsed_args, "upgrade-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
-            "overcloud-host-image-workaround-cloud-init",
             "seed-introspection-rules",
             "dell-switch-bmp")
         self.run_kayobe_playbooks(parsed_args, playbooks)
@@ -896,8 +940,9 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Configure the host's network interfaces.
     * Set sysctl parameters.
     * Disable bootstrap interface configuration.
-    * Configure NTP.
+    * Configure timezone.
     * Optionally, configure software RAID arrays.
+    * Optionally, configure encryption.
     * Configure LVM volumes.
     * Optionally, create a virtualenv for kolla-ansible.
     * Configure a user account for kolla-ansible.
@@ -916,23 +961,6 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Configuring overcloud host OS")
 
-        # Query some kayobe ansible variables.
-        # Explicitly request the dump-config tag to ensure this play runs even
-        # if the user specified tags.
-        hostvars = self.run_kayobe_config_dump(parsed_args, hosts="overcloud",
-                                               tags="dump-config")
-        if not hostvars:
-            self.app.LOG.error("No hosts in the overcloud group")
-            sys.exit(1)
-        hostvars = list(hostvars.values())[0]
-        ansible_user = hostvars.get("kayobe_ansible_user")
-        if not ansible_user:
-            self.app.LOG.error("Could not determine kayobe_ansible_user "
-                               "variable for overcloud hosts")
-            sys.exit(1)
-        python_interpreter = hostvars.get("ansible_python_interpreter")
-        kolla_target_venv = hostvars.get("kolla_ansible_target_venv")
-
         # Allocate IP addresses.
         playbooks = _build_playbook_list("ip-allocation")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
@@ -940,46 +968,25 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Kayobe playbooks.
         playbooks = _build_playbook_list(
             "ssh-known-host", "kayobe-ansible-user",
-            "yum", "dnf", "pip", "kayobe-target-venv")
+            "dnf", "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "disable-glean", "disable-cloud-init", "ntp", "mdadm",
-            "lvm", "docker-devicemapper")
+            "sysctl", "disable-glean", "disable-cloud-init", "timezone",
+            "mdadm", "luks", "lvm", "docker-devicemapper",
+            "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
         self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
         # Kolla-ansible bootstrap-servers.
-        # The kolla-ansible bootstrap-servers command should be run as the
-        # kayobe ansible user because at this point the kolla user may not
-        # exist.
-        extra_vars = {"ansible_user": ansible_user}
-        if python_interpreter:
-            # Use the kayobe virtualenv, as this is the executing user.
-            extra_vars["ansible_python_interpreter"] = python_interpreter
-        elif kolla_target_venv:
-            # Override the kolla-ansible virtualenv, use the system python
-            # instead.
-            extra_vars["ansible_python_interpreter"] = "/usr/bin/python"
-        if kolla_target_venv:
-            # Specify a virtualenv in which to install python packages.
-            extra_vars["virtualenv"] = kolla_target_venv
-        self.run_kolla_ansible_overcloud(parsed_args, "bootstrap-servers",
-                                         extra_vars=extra_vars)
-
-        # Re-run the Pip role after we've bootstrapped the Kolla user
-        extra_vars = {}
-        kolla_ansible_user = hostvars.get("kolla_ansible_user")
-        extra_vars["pip_applicable_users"] = [kolla_ansible_user]
+        self.run_kolla_ansible_overcloud(parsed_args, "bootstrap-servers")
 
         # Further kayobe playbooks.
         playbooks = _build_playbook_list(
-            "pip", "kolla-target-venv", "kolla-host",
-            "docker", "ceph-block-devices", "swift-block-devices")
-        self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars, limit="overcloud")
+            "kolla-host", "docker", "swift-block-devices")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
 
 class OvercloudHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
@@ -1041,7 +1048,8 @@ class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
 
-class OvercloudDatabaseBackup(KollaAnsibleMixin, VaultMixin, Command):
+class OvercloudDatabaseBackup(KollaAnsibleMixin, KayobeAnsibleMixin,
+                              VaultMixin, Command):
     """Backup the overcloud database."""
 
     def get_parser(self, prog_name):
@@ -1061,7 +1069,8 @@ class OvercloudDatabaseBackup(KollaAnsibleMixin, VaultMixin, Command):
                                          extra_args=extra_args)
 
 
-class OvercloudDatabaseRecover(KollaAnsibleMixin, VaultMixin, Command):
+class OvercloudDatabaseRecover(KollaAnsibleMixin, KayobeAnsibleMixin,
+                               VaultMixin, Command):
     """Recover the overcloud database."""
 
     def get_parser(self, prog_name):
@@ -1369,6 +1378,7 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
       deployment.
     * Perform a kolla-ansible upgrade of the overcloud services.
     * Configure and upgrade kayobe extra services.
+    * Regenerate openrc files for the admin user.
 
     This can be used in conjunction with the --tags and --kolla-tags arguments
     to upgrade specific services.
@@ -1400,6 +1410,13 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars, limit="overcloud")
 
+        # Post-deployment configuration.
+        self.run_kolla_ansible_overcloud(parsed_args, "post-deploy")
+        # Create an environment file for accessing the public API as the admin
+        # user.
+        playbooks = _build_playbook_list("public-openrc")
+        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
+
 
 class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
                               VaultMixin, Command):
