@@ -17,12 +17,13 @@ import glob
 import itertools
 import logging
 import os
-import six
 import subprocess
 import sys
 
 import yaml
 
+from kayobe import exception
+
 
 LOG = logging.getLogger(__name__)
 
@@ -70,18 +71,6 @@ def _get_base_path():
     return os.path.join(os.path.realpath(__file__), "..")
 
 
-def yum_install(packages):
-    """Install a list of packages via Yum."""
-    cmd = ["sudo", "yum", "-y", "install"]
-    cmd += packages
-    try:
-        run_command(cmd)
-    except subprocess.CalledProcessError as e:
-        print("Failed to install packages %s via Yum: returncode %d" %
-              (", ".join(packages), e.returncode))
-        sys.exit(e.returncode)
-
-
 def galaxy_install(role_file, roles_path, force=False):
     """Install Ansible roles via Ansible Galaxy."""
     cmd = ["ansible-galaxy", "install"]
@@ -163,7 +152,7 @@ def run_command(cmd, quiet=False, check_output=False, **kwargs):
     :param check_output: Whether to return the output of the command
     :returns: The output of the command if check_output is true
     """
-    if isinstance(cmd, six.string_types):
+    if isinstance(cmd, str):
         cmd_string = cmd
     else:
         cmd_string = " ".join(cmd)
@@ -190,7 +179,7 @@ def quote_and_escape(value):
     :param value: the string to quote and escape.
     :returns: the quoted and escaped string.
     """
-    if not isinstance(value, six.string_types):
+    if not isinstance(value, str):
         return value
     return "'" + value.replace("'", "'\\''") + "'"
 
@@ -208,3 +197,28 @@ def escape_jinja(string):
 
     b64_value = base64.b64encode(string.encode())
     return ''.join(('{{', "'", b64_value.decode(), "' | b64decode ", '}}'))
+
+
+def intersect_limits(args_limit, cli_limit):
+    """Create an Ansible host pattern of the intersection of two patterns.
+
+    :param args_limit: user-specified limit, or None.
+    :param cli_limit: limit originating from this CLI, or None.
+    :returns: a string representing an intersection of the two patterns.
+    """
+    # NOTE(mgoddard): Ansible uses either commas (,) or colons (:) to separate
+    # parts of a host pattern. An intersection is specified using a separator
+    # followed by an ampersand (&). If a mix of comma and colon separators is
+    # used, Ansible picks one and treats the other as part of the host pattern.
+    # This leads to hard to diagnose errors. Try to determine which separator
+    # the user has specified, and be consistent. Error if both are used.
+    if args_limit and ',' in args_limit:
+        if ':' in args_limit:
+            raise exception.Error("Invalid format for host limit argument. "
+                                  "Cannot mix commas and colons to separate "
+                                  "hosts")
+        separator = ',&'
+    else:
+        separator = ':&'
+    limits = [l for l in [args_limit, cli_limit] if l]
+    return separator.join(limits)
